"""Constants for the Dynalite component."""
import logging

LOGGER = logging.getLogger(__package__)
DOMAIN = "dynalite"
DATA_CONFIGS = "dynalite_configs"

CONF_BRIDGES = "bridges"

DEFAULT_NAME = "dynalite"
DEFAULT_PORT = 12345
